###install.packages('rsconnect')
###rsconnect::setAccountInfo(name='ayumi0201', token='76A72DCB8BA8FEAB92ABEE07D7C67BF5', secret='eTeD8uS9R5iytPMgOsoidHv/8jc5nqyhrlmXtInv')

#install.packages('shiny')
#install.packages('Rcpp')
#install.packages ('shinydashboard')
#install.packages('shinyWidgets')

library(shiny)
library(shinydashboard)
library(shinyWidgets)
library(rsconnect)

### load data from excel
library(readxl)

defaults <- read_excel("Coronavirus Cost 2020_05_05.xlsx", 
                            sheet = "QALY Calculations", range = "A7:F16")

### Make sure your current working directory is the folder with the excel sheet and app.R (this R script)
### If not, setwd() to where the file is with the directory name inside the parentheses. 


################################# User Interface ###################################
ui <- dashboardPage(
  dashboardHeader(title = "Choose Variables", titleWidth = 300),
  
  dashboardSidebar(
    actionButton(inputId = "go", 
                 "Calculate",
                 width = 270,
                 class = "btn-warning"),
    numericInput(inputId = "QALY_value", 
                 "Quality Adjusted Life Year Value ($)", 
                 value = 125000, min = 0),
    numericInput(inputId = "unmitigated_deaths",
                 "Unmitigated scenario: Projected COVID-19 deaths",
                 value = 2037423, min = 0),
    numericInput(inputId = "non_Covid_deaths_unmitigated",
                 "Unmitigated scenario: Additional non-COVID-19 deaths
                 due to system overload or avoidance",
                 value = 0, min = 0),
    numericInput(inputId = "present_deaths", 
                 "Present Scenario: Projected COVID-19 deaths", 
                 value = 122855, min = 0), 
    numericInput(inputId = "non_Covid_deaths",
                 "Present Scenario: Additional non-COVID-19 deaths (due to system overload or avoidance)",
                 value = 0, min = 0), 
    numericInput(inputId = "decrease_GDP_suppression",
                 "Per period decrease in GDP under suppression (%)",
                 value = 11.6, min = 0, max = 100), 
    numericInput(inputId = "decrease_GDP_test",
                 "Per period decrease in GDP under test, trace, and isolate (%)",
                 value = 7.9, min = 0, max = 100), 
    numericInput(inputId = "decrease_GDP_unmitigated",
                 "Per period decrease in GDP under unmitigated scenario prior to herd immunity (%)",
                 value = 7.9, min = 0, max = 100), 
    numericInput(inputId = "months_herd",
                 "Months before herd immunity achieved under unmitigated scenario",
                 value = 2.5, min = 0),
    numericInput(inputId = "months_stable",
                 "Months between herd immunity and GDP stabilizing",
                 value= 3.0, min = 0),
    numericInput(inputId = "months_suppression",
                 "Months before suppression achieved",
                 value = 2.5, min = 0),
    numericInput(inputId = "percent_male",
                 "Share of COVID-19 mortalities that are male (%)",
                 value = 75, min = 0),
    width = 300
  ),
  
  
  dashboardBody(
    fluidRow(h1("　Estimated Results")),
    fluidRow(valueBoxOutput(outputId = "vbox1", width = 12)),
    fluidRow(valueBoxOutput(outputId = "vbox2", width = 12)),
    tableOutput(outputId= 'table')
  )
)

################################# Server ###################################

# Define server logic required to provide outputs
server <- function(input, output) {
  
  ################################# Calculations ################################
  
  ### Default values (from May 5th, 2020 ver. excel sheet) ###
  fatality_rates_default <- defaults[[2]]
  popu_ratios <- defaults[[3]]
  avg_life_expectancy_male <- defaults[[5]] 
  avg_life_expectancy_female <- defaults[[6]]
  US_pop = 308710770
  baseline_GDP = 21.73 # trillion USD
  
  
  ### Reactive values ###
  
  # average life expectancy
  avg_life_expectancy <- reactive({
    avg_life_expectancy_male * input$percent_male / 100 +  avg_life_expectancy_female * (1 - input$percent_male / 100)
  })

  # US age-adjusted COVID-19 Fatality Rate 
  overall_fatality_rate <- reactive({
    crossprod(
      fatality_rates_default,
      popu_ratios
    )*100
  })
  
  # Before: Lost QALY per COVID-19 infection (Expected)
  # Update: US age- and sex-adjusted average reduced QALY from COVID-19 fatalities
  lost_QALY_per_infection <- reactive({
    sum(avg_life_expectancy()
        *popu_ratios
        *fatality_rates_default)
  })
  
  # Before: Lost QALY per COVID-19 infection (Expected) - in dollars
  # Update: US age- and sex-adjusted average monetary value of reduced QALY from COVID-19 fatalities PER INFECTION
  lost_QALY_per_infection_VALUE <- reactive({
    lost_QALY_per_infection()*input$QALY_value
  })
  
  # New: US age- and sex-adjusted average monetary value of reduced QALY from COVID-19 fatalities PER DEATH
  lost_QALY_per_death_VALUE <- reactive({
    lost_QALY_per_infection_VALUE()/overall_fatality_rate()
  })
  
  # Share of COVID-19 deaths
  # share_of_Covid_deaths <- reactive({
  #   c(popu_ratios*fatality_rates_default)/as.vector(crossprod(popu_ratios, fatality_rates_default))*100
  # })
  
  # Expected lost QALY from COVID-19 Deaths (present value) - UNMITIGATED
  lost_QALY_Covid_unmitigated <- reactive({
    lost_QALY_per_infection()*input$unmitigated_deaths/overall_fatality_rate()*100
  })
  
  # Expected lost QALY from COVID-19 Deaths (present value) - PRESENT STRATEGY
  lost_QALY_Covid <- reactive({
    lost_QALY_per_infection()*input$present_deaths/overall_fatality_rate()*100
  })
  
  # Before: Expected lost QALY from non-VOVID-19 deaths - UNMITIGATED
  # lost_QALY_non_Covid_unmitigated <- reactive({
  #   input$non_Covid_deaths_unmitigated*
  #     crossprod(
  #       avg_life_expectancy(),
  #       share_of_Covid_deaths()
  #     )
  # })
  
  # Update: Expected lost QALY from non-COVID-19 deaths - UNMITIGATED(present value)
  lost_QALY_non_Covid_unmitigated <- reactive({
    lost_QALY_per_infection()*input$non_Covid_deaths_unmitigated/overall_fatality_rate()*100
  })
  
  # Update: Expected lost QALY from non-COVID-19 deaths - PRESENT STRATEGY (present value)
  lost_QALY_non_Covid <- reactive({
    lost_QALY_per_infection()*input$non_Covid_deaths/overall_fatality_rate()*100
  })
  
  # Total Expected Lost QALY (present value) - UNMITIGATED
  total_lost_QALY_unmitigated <- reactive({
    lost_QALY_Covid_unmitigated() + lost_QALY_non_Covid_unmitigated()
  })
  
  # Total Expected Lost QALY (present value) - PRESENT STRATEGY
  total_lost_QALY <- reactive({
    lost_QALY_Covid() + lost_QALY_non_Covid()
  })
  
  # QALY savings over unmitigated
  QALY_savings <- reactive({
    total_lost_QALY_unmitigated() - total_lost_QALY()
  })
  
  # Expected total lost QALY (present value, billions USD) - Unmitigated
  expected_total_lost_QALY_unmitigated <- reactive({
    total_lost_QALY_unmitigated()*input$QALY_value/1000000000
  })
  
  # Expected total lost QALY (present value, billions USD) - Present Strategy
  expected_total_lost_QALY <- reactive({
    total_lost_QALY()*input$QALY_value/1000000000
  })
  
  # Monetary Value of QALY Gains Versus Unmitigated (billions, USD)
  QALY_savings_compared <- reactive({
    expected_total_lost_QALY_unmitigated() - expected_total_lost_QALY()
  })
  
  # Monetary Value of QALY Gained Versus Unmitigated Per Capita (USD)
  per_capita_QALY_savings_USD <- reactive({
    QALY_savings_compared()*1000000000/US_pop
  })
  
  # QALY Gained Versus Unmitigated Per Capita (days of life)
  per_capita_QALY_savings_days <- reactive({
    QALY_savings()/US_pop*365.25
  })
  
  # Lost QALY as total percent of GDP - Unmitigated
  lost_QALY_GDP_unmitigated <- reactive({
    expected_total_lost_QALY_unmitigated()/baseline_GDP/1000*100
  })
  
  # Lost QALY as total percent of GDP - Present Strategy
  lost_QALY_GDP <- reactive({
    expected_total_lost_QALY()/baseline_GDP/1000*100
  })
  
  
  # QALY Savings Over Unmitigated (percent of GDP)
  QALY_savings_compared_GDP <- reactive({
    lost_QALY_GDP_unmitigated()-lost_QALY_GDP()
  })
  
  ######
  
  # Update: Unmitigated Scenario Cost to GDP (percent of GDP)
  unmitigated_cost_GDP <- reactive({
    input$decrease_GDP_unmitigated*input$months_herd/12 + input$decrease_GDP_unmitigated/2*input$months_stable/12
  })

  
  # Update: Months of suppression alone before policy fails cost-benefit analysis
  months_suppression <- eventReactive(input$go, {
      (unmitigated_cost_GDP()+QALY_savings_compared_GDP())/(input$decrease_GDP_suppression/12)
    })
  
  # Update: Months of TTI before policy fails cost-benefit analysis (includes months required to achieve suppression)
  months_TTI <- eventReactive(input$go, {
    ((unmitigated_cost_GDP()+QALY_savings_compared_GDP())-(input$decrease_GDP_suppression/12*input$months_suppression))/(input$decrease_GDP_test/12)+input$months_suppression
  })
 
  ################################# Outputs ###################################
  
  table <- eventReactive(input$go, {
    data.frame(
      Name = c("Total expected lost QALY (present value)",
               "QALY savings over unmitigated",
               "Monetary Value of QALY Gains Versus Unmitigated (billions, USD)",
               "QALY savings over unmitigated (percent of GDP)",
               "Unmitigated Scenario Cost to GDP (percent of GDP)",
               "QALY Gained Versus Unmitigated Per Capita (days of life)",
               "Monetary Value of QALY Gained Versus Unmitigated Per Capita (USD)",
               "Months of suppression alone before policy fails cost-benefit analysis",
               "Months of TTI before policy fails cost-benefit analysis (includes months required to achieve suppression)"
      ),
      Value = prettyNum(
        c(
          total_lost_QALY(),
          QALY_savings(),
          round(QALY_savings_compared(),2),
          round(QALY_savings_compared_GDP(),1),
          round(unmitigated_cost_GDP(), 1),
          round(per_capita_QALY_savings_days(),1),
          round(per_capita_QALY_savings_USD()),
          round(months_suppression(),1),
          round(months_TTI(),1)
        ),
        big.mark = ","
      )
    )
  })
  
  output$vbox1 <- renderValueBox({
    valueBox(
      paste(round(months_suppression(),1), "months"),
      "Months of suppression alone before policy fails cost-benefit analysis", 
      color = "red",
      width = 6
    )
  })
  
  output$vbox2 <- renderValueBox({
    valueBox(
      paste(round(months_TTI(),1), "months"),
      "Months of TTI before policy fails cost-benefit analysis (includes months required to achieve suppression)", 
      color = "orange",
      width = 6
    )
  })
  
  
  output$table <- renderTable({
    table()
  })
  
}
################################# Make the shinyApp ###################################
shinyApp(ui = ui, server = server)
